test_that("determine if file is autogenerated", {
  expect_true(made_by_roxygen("made-by-roxygen/with-header.Rd"))
  expect_false(made_by_roxygen("made-by-roxygen/without-header.Rd"))
  expect_false(made_by_roxygen("made-by-roxygen/empty.Rd"))
  expect_true(check_made_by(made_by("#")))
  expect_false(check_made_by(character()))
})

test_that("roxygenize stops without side effects if no DESCRIPTION found", {
  path <- test_path("no-desc")
  expect_error(first_time(path), "must include a DESCRIPTION file")
})

